package admin.listener;


import admin.model.MaxKbChatReq;
import admin.model.WebSocketModel;
import admin.model.maxdb.req.MaxdbReq;
import admin.model.maxdb.resp.MaxdbStreamResponse;
import admin.modules.chatgpt.domain.ChatgptRoleRecords;
import admin.modules.chatgpt.service.ChatgptRoleRecordsService;
import admin.util.*;
import com.alibaba.fastjson.JSON;
import com.google.gson.Gson;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okhttp3.sse.EventSource;
import okhttp3.sse.EventSourceListener;

import javax.websocket.Session;
import java.io.IOException;
import java.net.SocketException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author lj
 * @date 2025/2/24 9:26
 */
@Slf4j
public class DeepSeekAiListener extends EventSourceListener {

    StringBuffer stringBuffer = new StringBuffer();
    private Set<String> set =new HashSet<>();
    private MaxKbChatReq maxKbChatReq;
    private ChatgptRoleRecordsService chatgptRoleRecordsService;

    public DeepSeekAiListener(MaxKbChatReq maxKbChatReq,ChatgptRoleRecordsService chatgptRoleRecordsService) {
        this.maxKbChatReq =maxKbChatReq;
        this.chatgptRoleRecordsService = chatgptRoleRecordsService;
    }

    @Override
    public void onOpen(EventSource eventSource, Response response) {
        super.onOpen(eventSource, response);
        //1.中科慧居简介直接发送
        sendStartTranslate(maxKbChatReq.getSession());
        log.info("DeepSeek建立sse连接...onOpen");
    }

    private void sendStartTranslate(Session session) {
        if (maxKbChatReq.getShowType()!=null && maxKbChatReq.getShowType().equals(ResponseEnum.ZK_PROFILE.getDesc())){
            WebSocketModel webSocketModel = new WebSocketModel();
            webSocketModel.setAction("start");
            webSocketModel.setType("text");
            webSocketModel.setShowType(maxKbChatReq.getShowType());
            webSocketModel.setContent("");
            try {
                session.getBasicRemote().sendText(JSON.toJSONString(webSocketModel));
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        } else if (maxKbChatReq.getShowType()!=null && maxKbChatReq.getShowType().equals(ResponseEnum.ZK_PRODUCT.getDesc())){
            WebSocketModel webSocketModel = new WebSocketModel();
            webSocketModel.setAction("start");
            webSocketModel.setType("text");
            webSocketModel.setShowType(maxKbChatReq.getShowType());
            webSocketModel.setContent("");
            try {
                session.getBasicRemote().sendText(JSON.toJSONString(webSocketModel));
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }else if (maxKbChatReq.getShowType()!=null && maxKbChatReq.getShowType().equals(ResponseEnum.ZK_USER.getDesc())){
            WebSocketModel webSocketModel = new WebSocketModel();
            webSocketModel.setAction("start");
            webSocketModel.setType("text");
            webSocketModel.setShowType(maxKbChatReq.getShowType());
            webSocketModel.setContent("");
            try {
                session.getBasicRemote().sendText(JSON.toJSONString(webSocketModel));
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }else if (maxKbChatReq.getShowType()!=null && maxKbChatReq.getShowType().equals(ResponseEnum.ZK_SOFTWARE.getDesc())){
            WebSocketModel webSocketModel = new WebSocketModel();
            webSocketModel.setAction("start");
            webSocketModel.setType("text");
            webSocketModel.setShowType(maxKbChatReq.getShowType());
            webSocketModel.setContent("");
            try {
                session.getBasicRemote().sendText(JSON.toJSONString(webSocketModel));
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }


    @Override
    public void onEvent(EventSource eventSource, String id, String type, String data) {
        super.onEvent(eventSource, id, type, data);
        //回复内容添加  sessionId
        Gson gson= new Gson();
        try {
            MaxdbStreamResponse completionResponse = gson.fromJson(data, MaxdbStreamResponse.class);
            if (completionResponse.getIsEnd()) {
                log.info("DeepSeek返回数据结束了...onEvent");
                Map<String,Object> map = new HashMap<>();
                map.put("action","done");
                map.put("type","text");
                map.put("showType",maxKbChatReq.getShowType());
                map.put("content",completionResponse.getContent());
                String msg = JSON.toJSONString(map);
                maxKbChatReq.getSession().getBasicRemote().sendText(msg);
                if (maxKbChatReq.getClient().equals(WebSocketUtil.H5)){
                    List<Session> webMap = WebConnectUtil.getWebMap(maxKbChatReq.getUserName());
                    if (!webMap.isEmpty()){
                        webMap.forEach(i->{
                            try {
                                i.getBasicRemote().sendText(msg);
                            } catch (IOException e) {
                                throw new RuntimeException(e);
                            }
                        });
                    }
                }

//                introduce();
                //存储聊天记录
                ChatgptRoleRecords chatgptRoleRecords = new ChatgptRoleRecords();
                chatgptRoleRecords.setMessage(this.stringBuffer.toString());
                chatgptRoleRecords.setReceiverId(ScreenEnum.getByUserName(maxKbChatReq.getUserName()));
                chatgptRoleRecords.setSessionId(Integer.valueOf(maxKbChatReq.getSessionId()));
                chatgptRoleRecords.setSenderId(10001);
                chatgptRoleRecords.setIsAsk(0);
                chatgptRoleRecords.setCreateTime(DateUtil.getNowTimestamp());
                chatgptRoleRecordsService.save(chatgptRoleRecords);
                return;
            }
            if (completionResponse.getContent()!=null){
                System.out.print(completionResponse.getContent());
                this.stringBuffer.append(completionResponse.getContent());
                Map<String,Object> map = new HashMap<>();
                map.put("action","reply");
                map.put("type","text");
                map.put("content",completionResponse.getContent());
                String msg = JSON.toJSONString(map);
                maxKbChatReq.getSession().getBasicRemote().sendText(msg);
                if (maxKbChatReq.getClient().equals(WebSocketUtil.H5)){
                    List<Session> webMap = WebConnectUtil.getWebMap(maxKbChatReq.getUserName());
                    //发送到web端展示动画
                    sendWebShowAnimation(webMap);
                    if (!webMap.isEmpty()){
                        webMap.forEach(i->{
                            try {
                                i.getBasicRemote().sendText(msg);
                            } catch (IOException e) {
                                throw new RuntimeException(e);
                            }
                        });
                    }
                }
            }
        }catch (Exception e){
            System.out.println(e.getMessage());
        }

//        ChatStreamResponse completionResponse = mapper.readValue(data, MaxdbStreamResponse.class);
//        if (completionResponse.getChoices()!=null && !completionResponse.getChoices().isEmpty()){
//            System.out.print(completionResponse.getChoices().get(0).getDelta().getContent());
//            this.stringBuffer.append(completionResponse.getChoices().get(0).getDelta().getContent());
//        }

    }

    private void sendWebShowAnimation(List<Session> webMap) {
        for (Session session : webMap) {
            if (!set.contains(session.getId())){
                set.add(session.getId());
                sendStartTranslate(session);
            }
        }
    }


    /**
     * 发送介绍信息(介绍产品信息 介绍人员信息)
     */
    private void introduce() {
        if (maxKbChatReq.getShowType().equals(ResponseEnum.ZK_PROFILE.getDesc())){
            maxKbChatReq.setMessage(ResponseEnum.ZK_PRODUCT.getDesc());
            maxKbChatReq.setShowType(ResponseEnum.ZK_PRODUCT.getDesc());
            MaxdbReq maxdbReq = new MaxdbReq();
            maxdbReq.setMessage(maxKbChatReq.getMessage());
            HttpUtil.postCallBackStream(maxKbChatReq.getMaxkbTalkUrl(),maxdbReq,maxKbChatReq.getMaxkbApiKey(),maxKbChatReq,chatgptRoleRecordsService);
        }else if (maxKbChatReq.getShowType().equals(ResponseEnum.ZK_PRODUCT.getDesc())){
            maxKbChatReq.setMessage(ResponseEnum.ZK_USER.getDesc());
            maxKbChatReq.setShowType(ResponseEnum.ZK_USER.getDesc());
            MaxdbReq maxdbReq = new MaxdbReq();
            maxdbReq.setMessage(maxKbChatReq.getMessage());
            HttpUtil.postCallBackStream(maxKbChatReq.getMaxkbTalkUrl(),maxdbReq,maxKbChatReq.getMaxkbApiKey(),maxKbChatReq,chatgptRoleRecordsService);
        }
    }

    @Override
    public void onClosed(EventSource eventSource) {
        super.onClosed(eventSource);
        log.info("DeepSeek关闭sse连接...onClosed");
    }

    @Override
    @SneakyThrows
    public void onFailure(EventSource eventSource, Throwable t, Response response) {
        super.onFailure(eventSource, t, response);
        if (Objects.isNull(response)) {
            return;
        }
        if(t instanceof SocketException){
            log.info("用户停止输出...onFailure");
        }else {
            ResponseBody body = response.body();
            if (Objects.nonNull(body)) {
                log.error("DeepSeek sse连接异常data：{}，异常：{}", body.string(), t);
            } else {
                log.error("DeepSeek sse连接异常data：{}，异常：{}", response, t);
            }
        }
        response.close();
        eventSource.cancel();
        log.info("eventSource cancel....onFailure");
    }
}
